@@ -1,16 +1,23 @@
+from __future__ import annotations
+
 from argparse import ArgumentParser
+from argparse import Namespace
+from configparser import ConfigParser
 import inspect
 import os
 import sys
+from typing import Dict
+from typing import Optional
+from typing import overload
+from typing import TextIO
 
 from . import __version__
 from . import command
 from . import util
 from .util import compat
-from .util.compat import SafeConfigParser
 
 
-class Config(object):
+class Config:
 
     r"""Represent an Alembic configuration.
 
@@ -72,7 +79,7 @@ class Config(object):
      is **copied** to a new one, stored locally as the attribute
      ``.config_args``. When the :attr:`.Config.file_config` attribute is
      first invoked, the replacement variable ``here`` will be added to this
-     dictionary before the dictionary is passed to ``SafeConfigParser()``
+     dictionary before the dictionary is passed to ``ConfigParser()``
      to parse the .ini file.
 
     :param attributes: optional dictionary of arbitrary Python keys/values,
@@ -86,14 +93,14 @@ class Config(object):
 
     def __init__(
         self,
-        file_=None,
-        ini_section="alembic",
-        output_buffer=None,
-        stdout=sys.stdout,
-        cmd_opts=None,
-        config_args=util.immutabledict(),
-        attributes=None,
-    ):
+        file_: Optional[str] = None,
+        ini_section: str = "alembic",
+        output_buffer: Optional[TextIO] = None,
+        stdout: TextIO = sys.stdout,
+        cmd_opts: Optional[Namespace] = None,
+        config_args: util.immutabledict = util.immutabledict(),
+        attributes: Optional[dict] = None,
+    ) -> None:
         """Construct a new :class:`.Config`"""
         self.config_file_name = file_
         self.config_ini_section = ini_section
@@ -104,7 +111,7 @@ class Config(object):
         if attributes:
             self.attributes.update(attributes)
 
-    cmd_opts = None
+    cmd_opts: Optional[Namespace] = None
     """The command-line options passed to the ``alembic`` script.
 
     Within an ``env.py`` script this can be accessed via the
@@ -116,14 +123,14 @@ class Config(object):
 
     """
 
-    config_file_name = None
+    config_file_name: Optional[str] = None
     """Filesystem path to the .ini file in use."""
 
-    config_ini_section = None
+    config_ini_section: str = None  # type:ignore[assignment]
     """Name of the config file section to read basic configuration
     from.  Defaults to ``alembic``, that is the ``[alembic]`` section
     of the .ini file.  This value is modified using the ``-n/--name``
-    option to the Alembic runnier.
+    option to the Alembic runner.
 
     """
 
@@ -147,7 +154,7 @@ class Config(object):
         """
         return {}
 
-    def print_stdout(self, text, *arg):
+    def print_stdout(self, text: str, *arg) -> None:
         """Render a message to standard out.
 
         When :meth:`.Config.print_stdout` is called with additional args
@@ -162,9 +169,9 @@ class Config(object):
         """
 
         if arg:
-            output = compat.text_type(text) % arg
+            output = str(text) % arg
         else:
-            output = compat.text_type(text)
+            output = str(text)
 
         util.write_outstream(self.stdout, output, "\n")
 
@@ -184,14 +191,14 @@ class Config(object):
         else:
             here = ""
         self.config_args["here"] = here
-        file_config = SafeConfigParser(self.config_args)
+        file_config = ConfigParser(self.config_args)
         if self.config_file_name:
             file_config.read([self.config_file_name])
         else:
             file_config.add_section(self.config_ini_section)
         return file_config
 
-    def get_template_directory(self):
+    def get_template_directory(self) -> str:
         """Return the directory where Alembic setup templates are found.
 
         This method is used by the alembic ``init`` and ``list_templates``
@@ -203,7 +210,19 @@ class Config(object):
         package_dir = os.path.abspath(os.path.dirname(alembic.__file__))
         return os.path.join(package_dir, "templates")
 
-    def get_section(self, name, default=None):
+    @overload
+    def get_section(
+        self, name: str, default: Dict[str, str]
+    ) -> Dict[str, str]:
+        ...
+
+    @overload
+    def get_section(
+        self, name: str, default: Optional[Dict[str, str]] = ...
+    ) -> Optional[Dict[str, str]]:
+        ...
+
+    def get_section(self, name: str, default=None):
         """Return all the configuration options from a given .ini file section
         as a dictionary.
 
@@ -213,7 +232,7 @@ class Config(object):
 
         return dict(self.file_config.items(name))
 
-    def set_main_option(self, name, value):
+    def set_main_option(self, name: str, value: str) -> None:
         """Set an option programmatically within the 'main' section.
 
         This overrides whatever was in the .ini file.
@@ -230,10 +249,10 @@ class Config(object):
         """
         self.set_section_option(self.config_ini_section, name, value)
 
-    def remove_main_option(self, name):
+    def remove_main_option(self, name: str) -> None:
         self.file_config.remove_option(self.config_ini_section, name)
 
-    def set_section_option(self, section, name, value):
+    def set_section_option(self, section: str, name: str, value: str) -> None:
         """Set an option programmatically within the given section.
 
         The section is created if it doesn't exist already.
@@ -257,7 +276,9 @@ class Config(object):
             self.file_config.add_section(section)
         self.file_config.set(section, name, value)
 
-    def get_section_option(self, section, name, default=None):
+    def get_section_option(
+        self, section: str, name: str, default: Optional[str] = None
+    ) -> Optional[str]:
         """Return an option from the given section of the .ini file."""
         if not self.file_config.has_section(section):
             raise util.CommandError(
@@ -269,6 +290,16 @@ class Config(object):
         else:
             return default
 
+    @overload
+    def get_main_option(self, name: str, default: str) -> str:
+        ...
+
+    @overload
+    def get_main_option(
+        self, name: str, default: Optional[str] = None
+    ) -> Optional[str]:
+        ...
+
     def get_main_option(self, name, default=None):
         """Return an option from the 'main' section of the .ini file.
 
@@ -280,11 +311,11 @@ class Config(object):
         return self.get_section_option(self.config_ini_section, name, default)
 
 
-class CommandLine(object):
-    def __init__(self, prog=None):
+class CommandLine:
+    def __init__(self, prog: Optional[str] = None) -> None:
         self._generate_args(prog)
 
-    def _generate_args(self, prog):
+    def _generate_args(self, prog: Optional[str]) -> None:
         def add_options(fn, parser, positional, kwargs):
             kwargs_opts = {
                 "template": (
@@ -491,8 +522,8 @@ class CommandLine(object):
                 and fn.__module__ == "alembic.command"
             ):
 
-                spec = compat.inspect_getargspec(fn)
-                if spec[3]:
+                spec = compat.inspect_getfullargspec(fn)
+                if spec[3] is not None:
                     positional = spec[0][1 : -len(spec[3])]
                     kwarg = spec[0][-len(spec[3]) :]
                 else:
@@ -515,7 +546,7 @@ class CommandLine(object):
                         else:
                             help_text.append(line.strip())
                 else:
-                    help_text = ""
+                    help_text = []
                 subparser = subparsers.add_parser(
                     fn.__name__, help=" ".join(help_text)
                 )
@@ -523,14 +554,14 @@ class CommandLine(object):
                 subparser.set_defaults(cmd=(fn, positional, kwarg))
         self.parser = parser
 
-    def run_cmd(self, config, options):
+    def run_cmd(self, config: Config, options: Namespace) -> None:
         fn, positional, kwarg = options.cmd
 
         try:
             fn(
                 config,
                 *[getattr(options, k, None) for k in positional],
-                **dict((k, getattr(options, k, None)) for k in kwarg)
+                **{k: getattr(options, k, None) for k in kwarg},
             )
         except util.CommandError as e:
             if options.raiseerr:
