@@ -40,11 +40,12 @@ class RevisionPathTest(MigrationTest):
     @classmethod
     def setup_class(cls):
         cls.env = env = staging_env()
-        cls.a = env.generate_revision(util.rev_id(), "->a")
-        cls.b = env.generate_revision(util.rev_id(), "a->b")
-        cls.c = env.generate_revision(util.rev_id(), "b->c")
-        cls.d = env.generate_revision(util.rev_id(), "c->d")
-        cls.e = env.generate_revision(util.rev_id(), "d->e")
+
+        cls.a = env.generate_revision("e6239818bb3a", "->a")
+        cls.b = env.generate_revision("548bbb905360", "a->b")
+        cls.c = env.generate_revision("b7ea43dc85e4", "b->c")
+        cls.d = env.generate_revision("1bbe33445780", "c->d")
+        cls.e = env.generate_revision("3975fb1a0125", "d->e")
 
     @classmethod
     def teardown_class(cls):
@@ -74,14 +75,14 @@ class RevisionPathTest(MigrationTest):
             self.e.revision,
             self.c.revision,
             [self.up_(self.d), self.up_(self.e)],
-            set([self.e.revision]),
+            {self.e.revision},
         )
 
         self._assert_upgrade(
             self.c.revision,
             None,
             [self.up_(self.a), self.up_(self.b), self.up_(self.c)],
-            set([self.c.revision]),
+            {self.c.revision},
         )
 
     def test_relative_upgrade_path(self):
@@ -89,32 +90,32 @@ class RevisionPathTest(MigrationTest):
             "+2",
             self.a.revision,
             [self.up_(self.b), self.up_(self.c)],
-            set([self.c.revision]),
+            {self.c.revision},
         )
 
         self._assert_upgrade(
-            "+1", self.a.revision, [self.up_(self.b)], set([self.b.revision])
+            "+1", self.a.revision, [self.up_(self.b)], {self.b.revision}
         )
 
         self._assert_upgrade(
             "+3",
             self.b.revision,
             [self.up_(self.c), self.up_(self.d), self.up_(self.e)],
-            set([self.e.revision]),
+            {self.e.revision},
         )
 
         self._assert_upgrade(
             "%s+2" % self.b.revision,
             self.a.revision,
             [self.up_(self.b), self.up_(self.c), self.up_(self.d)],
-            set([self.d.revision]),
+            {self.d.revision},
         )
 
         self._assert_upgrade(
             "%s-2" % self.d.revision,
             self.a.revision,
             [self.up_(self.b)],
-            set([self.b.revision]),
+            {self.b.revision},
         )
 
     def test_invalid_relative_upgrade_path(self):
@@ -141,7 +142,7 @@ class RevisionPathTest(MigrationTest):
             self.c.revision,
             self.e.revision,
             [self.down_(self.e), self.down_(self.d)],
-            set([self.c.revision]),
+            {self.c.revision},
         )
 
         self._assert_downgrade(
@@ -154,28 +155,28 @@ class RevisionPathTest(MigrationTest):
     def test_relative_downgrade_path(self):
 
         self._assert_downgrade(
-            "-1", self.c.revision, [self.down_(self.c)], set([self.b.revision])
+            "-1", self.c.revision, [self.down_(self.c)], {self.b.revision}
         )
 
         self._assert_downgrade(
             "-3",
             self.e.revision,
             [self.down_(self.e), self.down_(self.d), self.down_(self.c)],
-            set([self.b.revision]),
+            {self.b.revision},
         )
 
         self._assert_downgrade(
             "%s+2" % self.a.revision,
             self.d.revision,
             [self.down_(self.d)],
-            set([self.c.revision]),
+            {self.c.revision},
         )
 
         self._assert_downgrade(
             "%s-2" % self.c.revision,
             self.d.revision,
             [self.down_(self.d), self.down_(self.c), self.down_(self.b)],
-            set([self.a.revision]),
+            {self.a.revision},
         )
 
     def test_invalid_relative_downgrade_path(self):
@@ -286,7 +287,7 @@ class BranchedPathTest(MigrationTest):
             self.d1.revision,
             self.b.revision,
             [self.up_(self.c1), self.up_(self.d1)],
-            set([self.d1.revision]),
+            {self.d1.revision},
         )
 
     def test_upgrade_multiple_branch(self):
@@ -302,7 +303,7 @@ class BranchedPathTest(MigrationTest):
                 self.up_(self.c1),
                 self.up_(self.d1),
             ],
-            set([self.d1.revision, self.d2.revision]),
+            {self.d1.revision, self.d2.revision},
         )
 
     def test_downgrade_multiple_branch(self):
@@ -316,7 +317,7 @@ class BranchedPathTest(MigrationTest):
                 self.down_(self.c2),
                 self.down_(self.b),
             ],
-            set([self.a.revision]),
+            {self.a.revision},
         )
 
     def test_relative_upgrade(self):
@@ -325,7 +326,7 @@ class BranchedPathTest(MigrationTest):
             "c2branch@head-1",
             self.b.revision,
             [self.up_(self.c2)],
-            set([self.c2.revision]),
+            {self.c2.revision},
         )
 
     def test_relative_downgrade_baseplus2(self):
@@ -339,7 +340,7 @@ class BranchedPathTest(MigrationTest):
                 self.down_(self.d2),
                 self.down_(self.c2),
             ],
-            set([self.b.revision]),
+            {self.b.revision},
         )
 
     def test_relative_downgrade_branchplus2(self):
@@ -352,7 +353,7 @@ class BranchedPathTest(MigrationTest):
             "c2branch@base+2",
             [self.d2.revision, self.d1.revision],
             [self.down_(self.d2), self.down_(self.c2)],
-            set([self.d1.revision]),
+            {self.d1.revision},
         )
 
     def test_relative_downgrade_branchplus3(self):
@@ -361,13 +362,13 @@ class BranchedPathTest(MigrationTest):
             self.c2.revision,
             [self.d2.revision, self.d1.revision],
             [self.down_(self.d2)],
-            set([self.d1.revision, self.c2.revision]),
+            {self.d1.revision, self.c2.revision},
         )
         self._assert_downgrade(
             "c2branch@base+3",
             [self.d2.revision, self.d1.revision],
             [self.down_(self.d2)],
-            set([self.d1.revision, self.c2.revision]),
+            {self.d1.revision, self.c2.revision},
         )
 
     # Old downgrade -1 behaviour depends on order of branch upgrades.
@@ -380,7 +381,7 @@ class BranchedPathTest(MigrationTest):
                 "-1",
                 [self.d2.revision, self.d1.revision],
                 [self.down_(self.d2)],
-                set([self.d1.revision, self.c2.revision]),
+                {self.d1.revision, self.c2.revision},
             )
 
     def test_downgrade_once_order_right_unbalanced(self):
@@ -389,7 +390,7 @@ class BranchedPathTest(MigrationTest):
                 "-1",
                 [self.c2.revision, self.d1.revision],
                 [self.down_(self.c2)],
-                set([self.d1.revision]),
+                {self.d1.revision},
             )
 
     def test_downgrade_once_order_left(self):
@@ -398,7 +399,7 @@ class BranchedPathTest(MigrationTest):
                 "-1",
                 [self.d1.revision, self.d2.revision],
                 [self.down_(self.d1)],
-                set([self.d2.revision, self.c1.revision]),
+                {self.d2.revision, self.c1.revision},
             )
 
     def test_downgrade_once_order_left_unbalanced(self):
@@ -407,7 +408,7 @@ class BranchedPathTest(MigrationTest):
                 "-1",
                 [self.c1.revision, self.d2.revision],
                 [self.down_(self.c1)],
-                set([self.d2.revision]),
+                {self.d2.revision},
             )
 
     def test_downgrade_once_order_left_unbalanced_labelled(self):
@@ -415,73 +416,73 @@ class BranchedPathTest(MigrationTest):
             "c1branch@-1",
             [self.d1.revision, self.d2.revision],
             [self.down_(self.d1)],
-            set([self.c1.revision, self.d2.revision]),
+            {self.c1.revision, self.d2.revision},
         )
 
     # Captures https://github.com/sqlalchemy/alembic/issues/765
 
     def test_downgrade_relative_order_right(self):
         self._assert_downgrade(
-            "{}-1".format(self.d2.revision),
+            f"{self.d2.revision}-1",
             [self.d2.revision, self.c1.revision],
             [self.down_(self.d2)],
-            set([self.c1.revision, self.c2.revision]),
+            {self.c1.revision, self.c2.revision},
         )
 
     def test_downgrade_relative_order_left(self):
         self._assert_downgrade(
-            "{}-1".format(self.d2.revision),
+            f"{self.d2.revision}-1",
             [self.c1.revision, self.d2.revision],
             [self.down_(self.d2)],
-            set([self.c1.revision, self.c2.revision]),
+            {self.c1.revision, self.c2.revision},
         )
 
     def test_downgrade_single_branch_c1branch(self):
         """Use branch label to specify the branch to downgrade."""
         self._assert_downgrade(
-            "c1branch@{}".format(self.b.revision),
+            f"c1branch@{self.b.revision}",
             (self.c1.revision, self.d2.revision),
             [
                 self.down_(self.c1),
             ],
-            set([self.d2.revision]),
+            {self.d2.revision},
         )
 
     def test_downgrade_single_branch_c1branch_from_d1_head(self):
         """Use branch label to specify the branch (where the branch label is
         not on the head revision)."""
         self._assert_downgrade(
-            "c2branch@{}".format(self.b.revision),
+            f"c2branch@{self.b.revision}",
             (self.c1.revision, self.d2.revision),
             [
                 self.down_(self.d2),
                 self.down_(self.c2),
             ],
-            set([self.c1.revision]),
+            {self.c1.revision},
         )
 
     def test_downgrade_single_branch_c2(self):
         """Use a revision on the branch (not head) to specify the branch."""
         self._assert_downgrade(
-            "{}@{}".format(self.c2.revision, self.b.revision),
+            f"{self.c2.revision}@{self.b.revision}",
             (self.d1.revision, self.d2.revision),
             [
                 self.down_(self.d2),
                 self.down_(self.c2),
             ],
-            set([self.d1.revision]),
+            {self.d1.revision},
         )
 
     def test_downgrade_single_branch_d1(self):
         """Use the head revision to specify the branch."""
         self._assert_downgrade(
-            "{}@{}".format(self.d1.revision, self.b.revision),
+            f"{self.d1.revision}@{self.b.revision}",
             (self.d1.revision, self.d2.revision),
             [
                 self.down_(self.d1),
                 self.down_(self.c1),
             ],
-            set([self.d2.revision]),
+            {self.d2.revision},
         )
 
     def test_downgrade_relative_to_branch_head(self):
@@ -489,7 +490,7 @@ class BranchedPathTest(MigrationTest):
             "c1branch@head-1",
             (self.d1.revision, self.d2.revision),
             [self.down_(self.d1)],
-            set([self.c1.revision, self.d2.revision]),
+            {self.c1.revision, self.d2.revision},
         )
 
     def test_upgrade_other_branch_from_mergepoint(self):
@@ -499,7 +500,7 @@ class BranchedPathTest(MigrationTest):
             "c2branch@+1",
             (self.c1.revision),
             [self.up_(self.c2)],
-            set([self.c1.revision, self.c2.revision]),
+            {self.c1.revision, self.c2.revision},
         )
 
     def test_upgrade_one_branch_of_heads(self):
@@ -510,7 +511,7 @@ class BranchedPathTest(MigrationTest):
             "c2branch@+1",
             (self.c1.revision, self.c2.revision),
             [self.up_(self.d2)],
-            set([self.c1.revision, self.d2.revision]),
+            {self.c1.revision, self.d2.revision},
         )
 
     def test_ambiguous_upgrade(self):
@@ -524,13 +525,11 @@ class BranchedPathTest(MigrationTest):
 
     def test_upgrade_from_base(self):
         self._assert_upgrade(
-            "base+1", [], [self.up_(self.a)], set([self.a.revision])
+            "base+1", [], [self.up_(self.a)], {self.a.revision}
         )
 
     def test_upgrade_from_base_implicit(self):
-        self._assert_upgrade(
-            "+1", [], [self.up_(self.a)], set([self.a.revision])
-        )
+        self._assert_upgrade("+1", [], [self.up_(self.a)], {self.a.revision})
 
     def test_downgrade_minus1_to_base(self):
         self._assert_downgrade(
@@ -552,13 +551,13 @@ class BranchedPathTest(MigrationTest):
             self.c2.revision,
             [self.d1.revision, self.c2.revision],
             [],
-            set([self.d1.revision, self.c2.revision]),
+            {self.d1.revision, self.c2.revision},
         )
         self._assert_downgrade(
             self.d1.revision,
             [self.d1.revision, self.c2.revision],
             [],
-            set([self.d1.revision, self.c2.revision]),
+            {self.d1.revision, self.c2.revision},
         )
 
 
@@ -613,7 +612,7 @@ class BranchFromMergepointTest(MigrationTest):
             self.d1.revision,
             (self.d2.revision, self.b1.revision),
             [self.up_(self.c1), self.up_(self.d1)],
-            set([self.d2.revision, self.d1.revision]),
+            {self.d2.revision, self.d1.revision},
         )
 
     def test_mergepoint_to_only_one_side_downgrade(self):
@@ -622,7 +621,7 @@ class BranchFromMergepointTest(MigrationTest):
             self.b1.revision,
             (self.d2.revision, self.d1.revision),
             [self.down_(self.d1), self.down_(self.c1)],
-            set([self.d2.revision, self.b1.revision]),
+            {self.d2.revision, self.b1.revision},
         )
 
 
@@ -697,7 +696,7 @@ class BranchFrom3WayMergepointTest(MigrationTest):
             self.d1.revision,
             (self.d3.revision, self.d2.revision, self.b1.revision),
             [self.up_(self.c1), self.up_(self.d1)],
-            set([self.d3.revision, self.d2.revision, self.d1.revision]),
+            {self.d3.revision, self.d2.revision, self.d1.revision},
         )
 
     def test_mergepoint_to_only_one_side_downgrade(self):
@@ -705,7 +704,7 @@ class BranchFrom3WayMergepointTest(MigrationTest):
             self.b1.revision,
             (self.d3.revision, self.d2.revision, self.d1.revision),
             [self.down_(self.d1), self.down_(self.c1)],
-            set([self.d3.revision, self.d2.revision, self.b1.revision]),
+            {self.d3.revision, self.d2.revision, self.b1.revision},
         )
 
     def test_mergepoint_to_two_sides_upgrade(self):
@@ -715,7 +714,7 @@ class BranchFrom3WayMergepointTest(MigrationTest):
             (self.d3.revision, self.b2.revision, self.b1.revision),
             [self.up_(self.c2), self.up_(self.c1), self.up_(self.d1)],
             # this will merge b2 and b1 into d1
-            set([self.d3.revision, self.d1.revision]),
+            {self.d3.revision, self.d1.revision},
         )
 
         # but then!  b2 will break out again if we keep going with it
@@ -723,7 +722,7 @@ class BranchFrom3WayMergepointTest(MigrationTest):
             self.d2.revision,
             (self.d3.revision, self.d1.revision),
             [self.up_(self.d2)],
-            set([self.d3.revision, self.d2.revision, self.d1.revision]),
+            {self.d3.revision, self.d2.revision, self.d1.revision},
         )
 
 
@@ -915,14 +914,14 @@ class DependsOnBranchTestOne(MigrationTest):
         heads = [self.c2.revision, self.d1.revision]
         head = HeadMaintainer(mock.Mock(), heads)
         head.update_to_step(self.down_(self.d1))
-        eq_(head.heads, set([self.c2.revision]))
+        eq_(head.heads, {self.c2.revision})
 
     def test_stamp_across_dependency(self):
         heads = [self.e1.revision, self.c2.revision]
         head = HeadMaintainer(mock.Mock(), heads)
         for step in self.env._stamp_revs(self.b1.revision, heads):
             head.update_to_step(step)
-        eq_(head.heads, set([self.b1.revision]))
+        eq_(head.heads, {self.b1.revision})
 
 
 class DependsOnBranchTestTwo(MigrationTest):
@@ -1009,15 +1008,13 @@ class DependsOnBranchTestTwo(MigrationTest):
             self.b2.revision,
             heads,
             [self.down_(self.bmerge)],
-            set(
-                [
-                    self.amerge.revision,
-                    self.b1.revision,
-                    self.cmerge.revision,
-                    # b2 isn't here, but d1 is, which implies b2. OK!
-                    self.d1.revision,
-                ]
-            ),
+            {
+                self.amerge.revision,
+                self.b1.revision,
+                self.cmerge.revision,
+                # b2 isn't here, but d1 is, which implies b2. OK!
+                self.d1.revision,
+            },
         )
 
         # start with those heads..
@@ -1033,15 +1030,13 @@ class DependsOnBranchTestTwo(MigrationTest):
             "d1@base",
             heads,
             [self.down_(self.d1)],
-            set(
-                [
-                    self.amerge.revision,
-                    self.b1.revision,
-                    # b2 has to be INSERTed, because it was implied by d1
-                    self.b2.revision,
-                    self.cmerge.revision,
-                ]
-            ),
+            {
+                self.amerge.revision,
+                self.b1.revision,
+                # b2 has to be INSERTed, because it was implied by d1
+                self.b2.revision,
+                self.cmerge.revision,
+            },
         )
 
         # start with those heads ...
@@ -1070,7 +1065,7 @@ class DependsOnBranchTestTwo(MigrationTest):
                 self.down_(self.c2),
                 self.down_(self.c3),
             ],
-            set([]),
+            set(),
         )
 
 
@@ -1121,7 +1116,7 @@ class DependsOnBranchTestThree(MigrationTest):
             "b1",
             ["a3", "b2"],
             [self.down_(self.b2)],
-            set(["a3"]),  # we have b1 also, which is implied by a3
+            {"a3"},  # we have b1 also, which is implied by a3
         )
 
 
@@ -1144,7 +1139,7 @@ class DependsOnOwnDownrevTest(MigrationTest):
             self.a2.revision,
             None,
             [self.up_(self.a1), self.up_(self.a2)],
-            set(["a2"]),
+            {"a2"},
         )
 
     def test_traverse_down(self):
@@ -1152,7 +1147,7 @@ class DependsOnOwnDownrevTest(MigrationTest):
             self.a1.revision,
             self.a2.revision,
             [self.down_(self.a2)],
-            set(["a1"]),
+            {"a1"},
         )
 
 
@@ -1189,7 +1184,7 @@ class DependsOnBranchTestFour(MigrationTest):
             heads,
             [self.down_(self.b4)],
             # a3 isn't here, because b3 still implies a3
-            set([self.b3.revision]),
+            {self.b3.revision},
         )
 
 
@@ -1238,7 +1233,7 @@ class DependsOnBranchLabelTest(MigrationTest):
                 self.up_(self.b2),
                 self.up_(self.c2),
             ],
-            set([self.c2.revision]),
+            {self.c2.revision},
         )
 
 
@@ -1275,8 +1270,8 @@ class ForestTest(MigrationTest):
         revs = self.env._stamp_revs("heads", ())
         eq_(len(revs), 2)
         eq_(
-            set(r.to_revisions for r in revs),
-            set([(self.b1.revision,), (self.b2.revision,)]),
+            {r.to_revisions for r in revs},
+            {(self.b1.revision,), (self.b2.revision,)},
         )
 
     def test_stamp_to_heads_no_moves_needed(self):
@@ -1447,19 +1442,19 @@ class BranchedPathTestCrossDependencies(MigrationTest):
         """c2branch depends on c1branch so can be taken down on its own.
         Current behaviour also takes down the dependency unnecessarily."""
         self._assert_downgrade(
-            "c2branch@{}".format(self.b.revision),
+            f"c2branch@{self.b.revision}",
             (self.d1.revision, self.d2.revision),
             [
                 self.down_(self.d2),
                 self.down_(self.c2),
             ],
-            set([self.d1.revision]),
+            {self.d1.revision},
         )
 
     def test_downgrade_branch_dependency(self):
         """c2branch depends on c1branch so taking down c1branch requires taking
         down both"""
-        destination = "c1branch@{}".format(self.b.revision)
+        destination = f"c1branch@{self.b.revision}"
         source = self.d1.revision, self.d2.revision
         revs = self.env._downgrade_revs(destination, source)
         # Drops c1, d1 as requested, also drops d2 due to dependence on d1.
@@ -1482,4 +1477,4 @@ class BranchedPathTestCrossDependencies(MigrationTest):
         head = HeadMaintainer(mock.Mock(), heads)
         for rev in revs:
             head.update_to_step(rev)
-        eq_(head.heads, set([self.c2.revision]))
+        eq_(head.heads, {self.c2.revision})
